type tp = Bool 
        | Nat 
        | Arr of (tp * tp)   (* Arr (ty1,ty2) = ty1 -> ty2 *)
        | Times of (tp * tp) (* Times (ty1,ty2) = ty1 * ty2 *)

type tm = Zero                   (* zero *)
        | Succ of tm             (* successor *)
        | Bval of bool           (* boolean values *)
        | Rel of int             (* de Bruijn index *)
        | Lam of (tp * tm)       (* lambda abstraction *)
        | App of (tm * tm)       (* application *)
        | Pair of (tm * tm)      (* pair *)
        | Fst of tm              (* first of a pair *)
        | Snd of tm              (* second of a pair *)
        | Disc of (tm * tm * tm) (* if-then-else *)
        | Rec of (tm * tm * tm)  (* recursion operator *)

val typecheck_T : tp list -> tm -> tp

val plus_T : tm
val times_T : tm
val ack_T : tm

val eval : tm -> tm
val nat_of_t : tm -> int
val t_of_nat : int -> tm
